@@ -3,10 +3,10 @@
 
   rbgdkevent.c -
 
-  $Author: sakai $
-  $Date: 2003/11/20 18:27:54 $
+  $Author: mutoh $
+  $Date: 2006/12/26 16:11:13 $
 
-  Copyright (C) 2002,2003 Ruby-GNOME2 Project Team
+  Copyright (C) 2002-2004 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
                           Daisuke Kanda,
                           Hiroshi Igarashi
@@ -14,7 +14,13 @@
 
 #include "global.h"
 
+#if GTK_CHECK_VERSION(2,8,0)
+static VALUE gdkevents[36];
+#elif GTK_CHECK_VERSION(2,6,0)
+static VALUE gdkevents[35];
+#else
 static VALUE gdkevents[34];
+#endif
 
 /***********************************************/
 
@@ -109,7 +115,13 @@ static VALUE \
 gdkevent ## type ## _set_ ## name (self, val)\
     VALUE self, val;\
 {\
-    get_gdkevent(self)->type.name = RVAL2GOBJ(val);\
+    GdkEvent *event;\
+    event = get_gdkevent(self);\
+    if (event->type.name)\
+      g_object_unref(event->type.name);\
+    event->type.name = RVAL2GOBJ(val);\
+    if (event->type.name)\
+      g_object_ref(event->type.name);\
     return self;\
 }
 
@@ -133,7 +145,8 @@ static VALUE \
 gdkevent ## type ## _ ## name (self)\
     VALUE self;\
 {\
-    return BOXED2RVAL(get_gdkevent(self)->type.name, GDK_TYPE_ATOM);\
+    GdkAtom atom = get_gdkevent(self)->type.name;\
+    return BOXED2RVAL(atom, GDK_TYPE_ATOM);\
 }\
 static VALUE \
 gdkevent ## type ## _set_ ## name (self, val)\
@@ -197,6 +210,45 @@ gdkevent ## type ## _set_axes(self, x, y)\
     rb_define_method(event, G_STRINGIFY(name), gdkevent ## type ## _## name, 0);\
     rb_define_method(event, G_STRINGIFY(set_ ## name), gdkevent ## type ## _set_## name, 1);
 
+
+/* initialize */
+#if GTK_CHECK_VERSION(2,2,0)
+#define GDKEVENT_INIT(type, default_gtype) \
+static VALUE \
+gdkevent ## type ## _initialize(argc, argv, self)\
+    int argc;\
+    VALUE *argv;\
+    VALUE self;\
+{\
+    VALUE type;\
+    GdkEventType gtype;\
+\
+    rb_scan_args(argc, argv, "01", &type);\
+    if (NIL_P(type)){\
+        gtype = default_gtype;\
+    } else {\
+        gtype = RVAL2GENUM(type, GDK_TYPE_EVENT_TYPE);\
+    }\
+\
+    G_INITIALIZE(self, gdk_event_new(gtype));\
+    return Qnil;\
+}
+#else
+#define GDKEVENT_INIT(type, default_gtype) \
+static VALUE \
+gdkevent ## type ## _initialize(argc, argv, self)\
+    int argc;\
+    VALUE *argv;\
+    VALUE self;\
+{\
+    rb_raise(rb_eRuntimeError, "Gdk::Event.new is not supported in this environment.");\
+    return Qnil;\
+}
+#endif
+
+#define DEFINE_INIT(event, type)                                         \
+  rb_define_method(event, "initialize", gdkevent ## type ## _initialize, -1);
+
 /***********************************************/
 
 /* GdkEvent Singleton Methods */
@@ -255,7 +307,7 @@ gdkevent_put(self)
     VALUE self;
 {
     gdk_event_put(get_gdkevent(self));
-    return Qnil;
+    return self;
 }
 
 /* We don't need this.
@@ -344,14 +396,33 @@ gdkevent_screen(self)
 }
 #endif
 
+/*
+  type: String, Integer, Gdk::Color.
+ */
 static VALUE
-gdkevent_s_setting_get(self, name)
-    VALUE self, name;
+gdkevent_s_setting_get(argc, argv, self)
+    int argc;
+    VALUE* argv;
+    VALUE self;
 {
-    GValue value = {0, };
+    VALUE name, type;
+    GType gtype;
+    GValue val = {0,};
+    gboolean ret;
+    VALUE value;
 
-    gboolean ret = gdk_setting_get(RVAL2CSTR(name), &value);
-    return ret ? GVAL2RVAL(&value) : Qnil;
+    rb_scan_args(argc, argv, "11", &name, &type);
+    if NIL_P(type) 
+        gtype = G_TYPE_STRING;
+    else
+        gtype = CLASS2GTYPE(type);
+
+    g_value_init(&val, gtype);
+    ret = gdk_setting_get(RVAL2CSTR(name), &val);
+
+    value = ret ? GVAL2RVAL(&val) : Qnil;
+    g_value_unset(&val);
+    return value;
 }
 
 /* GdkEventAny */
@@ -362,8 +433,6 @@ ATTR_BOOL(any, send_event);
 ATTR_UINT(key, time);
 ATTR_FLAGS(key, state, GDK_TYPE_MODIFIER_TYPE);
 ATTR_INT(key, keyval);
-ATTR_INT(key, length);
-ATTR_STR(key, string);
 ATTR_UINT(key, hardware_keycode);
 ATTR_UINT(key, group);
 
@@ -387,6 +456,7 @@ ATTR_ENUM(scroll, direction, GDK_TYPE_SCROLL_DIRECTION);
 ATTR_GOBJ(scroll, device);
 ATTR_FLOAT(scroll, x_root);
 ATTR_FLOAT(scroll, y_root);
+GDKEVENT_INIT(scroll, GDK_SCROLL);
 
 /* GdkEventMotion */
 ATTR_UINT(motion, time);
@@ -398,6 +468,7 @@ ATTR_FLOAT(motion, y_root);
 ATTR_FLAGS(motion, state, GDK_TYPE_MODIFIER_TYPE);
 ATTR_BOOL(motion, is_hint);
 ATTR_GOBJ(motion, device);
+GDKEVENT_INIT(motion, GDK_MOTION_NOTIFY);
 
 /* GdkEventExpose */
 static VALUE
@@ -435,9 +506,11 @@ gdkeventexpose_set_region(self, region)
 }
 
 ATTR_INT(expose, count);
+GDKEVENT_INIT(expose, GDK_EXPOSE);
 
 /* GdkEventVisibility */
 ATTR_ENUM(visibility, state, GDK_TYPE_VISIBILITY_STATE);
+GDKEVENT_INIT(visibility, GDK_VISIBILITY_NOTIFY);
 
 /* GdkEventCrossing */
 ATTR_GOBJ(crossing, subwindow);
@@ -453,17 +526,20 @@ ATTR_FLAGS(crossing, state, GDK_TYPE_MODIFIER_TYPE);
 
 /* GdkEventFocus */
 ATTR_BOOL(focus_change, in);
+GDKEVENT_INIT(focus_change, GDK_FOCUS_CHANGE);
 
 /* GdkEventConfigure */
 ATTR_INT(configure, x);
 ATTR_INT(configure, y);
 ATTR_INT(configure, width);
 ATTR_INT(configure, height);
+GDKEVENT_INIT(configure, GDK_CONFIGURE);
 
 /* GdkEventProperty */
 ATTR_ATOM(property, atom);
 ATTR_UINT(property, time);
 ATTR_ENUM(property, state, GDK_TYPE_PROPERTY_STATE);
+GDKEVENT_INIT(property, GDK_PROPERTY_NOTIFY);
 
 /* GdkEventSelection */
 ATTR_ATOM(selection, selection);
@@ -484,6 +560,7 @@ ATTR_GOBJ(proximity, device);
 
 /* GdkEventClient */
 ATTR_ATOM(client, message_type);
+GDKEVENT_INIT(client, GDK_CLIENT_EVENT);
 
 static VALUE
 gdkeventclient_data_format(self)
@@ -547,7 +624,7 @@ gdkeventclient_send_clientmessage_toall(self)
     VALUE self;
 {
     gdk_event_send_clientmessage_toall(get_gdkevent(self));
-    return Qnil;
+    return self;
 }
 
 static GdkFilterReturn
@@ -556,7 +633,7 @@ filter_func(xevent, event, func)
     GdkEvent* event;
     gpointer func;
 {
-    VALUE ret = rb_funcall((VALUE)func, id_call, 2, UINT2NUM((guint)xevent), make_gdkevent(event));
+    VALUE ret = rb_funcall((VALUE)func, id_call, 2, LONG2NUM((glong)xevent), make_gdkevent(event));
     return RVAL2GENUM(ret, GDK_TYPE_FILTER_RETURN);
 }
 
@@ -571,14 +648,36 @@ gdkevent_s_add_client_message_filter(self, message_type)
     return self;
 }
 
+/* GdkEventNoExpose */
+GDKEVENT_INIT(noexpose, GDK_NO_EXPOSE);
+
 /* GdkEventWindowState */
 ATTR_FLAGS(window_state, changed_mask, GDK_TYPE_WINDOW_STATE);
 ATTR_FLAGS(window_state, new_window_state, GDK_TYPE_WINDOW_STATE);
+GDKEVENT_INIT(window_state, GDK_WINDOW_STATE);
 
 /* GdkEventSetting */
 ATTR_ENUM(setting, action, GDK_TYPE_SETTING_ACTION);
 ATTR_STR(setting, name);
+GDKEVENT_INIT(setting, GDK_SETTING);
 
+/* GdkEventOwnerChange */
+#if GTK_CHECK_VERSION(2,6,0)
+ATTR_UINT(owner_change, owner);
+ATTR_ENUM(owner_change, reason, GDK_TYPE_OWNER_CHANGE);
+ATTR_ATOM(owner_change, selection);
+ATTR_UINT(owner_change, time);
+ATTR_UINT(owner_change, selection_time);
+GDKEVENT_INIT(owner_change, GDK_OWNER_CHANGE);
+#endif
+
+/* GdkEventGrabBroken */
+#if GTK_CHECK_VERSION(2,8,0)
+ATTR_BOOL(grab_broken, keyboard);
+ATTR_BOOL(grab_broken, implicit);
+ATTR_GOBJ(grab_broken, grab_window);
+GDKEVENT_INIT(grab_broken, GDK_GRAB_BROKEN);
+#endif
 
 /* MISC */
 static VALUE 
@@ -631,7 +730,12 @@ Init_gtk_gdk_event()
     gdkevents[GDK_SCROLL]        = rb_define_class_under(mGdk, "EventScroll", gdkEventAny);
     gdkevents[GDK_WINDOW_STATE]  = rb_define_class_under(mGdk, "EventWindowState", gdkEventAny);
     gdkevents[GDK_SETTING]       = rb_define_class_under(mGdk, "EventSetting", gdkEventAny);
-
+#if GTK_CHECK_VERSION(2,6,0)
+    gdkevents[GDK_OWNER_CHANGE]  = rb_define_class_under(mGdk, "EventOwnerChange", gdkEventAny);
+#endif
+#if GTK_CHECK_VERSION(2,8,0)
+    gdkevents[GDK_GRAB_BROKEN]   = rb_define_class_under(mGdk, "EventGrabBroken", gdkEventAny);
+#endif
 
     /* GdkEvent */
 #if GTK_CHECK_VERSION(2,2,0)
@@ -642,8 +746,6 @@ Init_gtk_gdk_event()
     rb_define_singleton_method(gdkEvent, "events_pending?", gdkevent_s_events_pending, 0);
     rb_define_singleton_method(gdkEvent, "peek", gdkevent_s_peek, 0);
     rb_define_singleton_method(gdkEvent, "get", gdkevent_s_get, 0);
-    rb_define_singleton_method(gdkEvent, "get_graphics_expose", 
-							   gdkevent_s_get_graphics_expose, 1);
     rb_define_method(gdkEvent, "put", gdkevent_put, 0);
     rb_define_method(gdkEvent, "get_axis", gdkevent_get_axis, 1);
     rb_define_method(gdkEvent, "coords", gdkevent_get_coords, 0);
@@ -652,27 +754,28 @@ Init_gtk_gdk_event()
     rb_define_singleton_method(gdkEvent, "handler_set", gdkevent_s_handler_set, 0);
     rb_define_singleton_method(gdkEvent, "show_events?", gdkevent_s_get_show_events, 0);
     rb_define_singleton_method(gdkEvent, "set_show_events", gdkevent_s_set_show_events, 1);
-    rb_define_singleton_method(gdkEvent, "setting_get", gdkevent_s_setting_get, 1);
+    rb_define_singleton_method(gdkEvent, "setting_get", gdkevent_s_setting_get, -1);
     rb_define_singleton_method(gdkEvent, "add_client_message_filter", gdkevent_s_add_client_message_filter, 1);
 #if GTK_CHECK_VERSION(2,2,0)
     rb_define_method(gdkEvent, "screen", gdkevent_screen, 0);
     rb_define_method(gdkEvent, "set_screen", gdkevent_set_screen, 1);
 #endif
+    G_DEF_SETTERS(gdkEvent);
 
     /* GdkEventAny */
-    DEFINE_ACCESSOR(gdkEvent, any, window);
+    DEFINE_ACCESSOR(gdkEventAny, any, window);
     rb_define_method(gdkEventAny, "send_event?", gdkeventany_send_event, 0);
     rb_define_method(gdkEventAny, "set_send_event", gdkeventany_set_send_event, 1);
+    G_DEF_SETTERS(gdkEventAny);
 
     /* GdkEventKey */
     ev = gdkevents[GDK_KEY_PRESS];
     DEFINE_ACCESSOR(ev, key, time);
     DEFINE_ACCESSOR(ev, key, state);
     DEFINE_ACCESSOR(ev, key, keyval);
-    DEFINE_ACCESSOR(ev, key, length);
-    DEFINE_ACCESSOR(ev, key, string);
     DEFINE_ACCESSOR(ev, key, hardware_keycode);
     DEFINE_ACCESSOR(ev, key, group);
+    G_DEF_SETTERS(ev);
 
     /* GdkEventButton */
     ev = gdkevents[GDK_BUTTON_PRESS];
@@ -686,6 +789,7 @@ Init_gtk_gdk_event()
     DEFINE_ACCESSOR(ev, button, device);
     DEFINE_ACCESSOR(ev, button, x_root);
     DEFINE_ACCESSOR(ev, button, y_root);
+    G_DEF_SETTERS(ev);
 
     /* GdkEventScroll */
     ev = gdkevents[GDK_SCROLL];
@@ -697,6 +801,8 @@ Init_gtk_gdk_event()
     DEFINE_ACCESSOR(ev, scroll, device);
     DEFINE_ACCESSOR(ev, scroll, x_root);
     DEFINE_ACCESSOR(ev, scroll, y_root);
+    DEFINE_INIT(ev, scroll);
+    G_DEF_SETTERS(ev);
 
     /* GdkScrollDirection */
     G_DEF_CLASS(GDK_TYPE_SCROLL_DIRECTION, "Direction", ev);
@@ -715,16 +821,24 @@ Init_gtk_gdk_event()
     DEFINE_ACCESSOR(ev, motion, device);
     DEFINE_ACCESSOR(ev, motion, x_root);
     DEFINE_ACCESSOR(ev, motion, y_root);
+    DEFINE_INIT(ev, motion);
+    G_DEF_SETTERS(ev);
 
     /* GdkEventExpose */
     ev = gdkevents[GDK_EXPOSE];
     DEFINE_ACCESSOR(ev, expose, area);
     DEFINE_ACCESSOR(ev, expose, region);
     DEFINE_ACCESSOR(ev, expose, count);
+    DEFINE_INIT(ev, expose);
+    rb_define_singleton_method(ev, "get_graphics_expose", 
+                               gdkevent_s_get_graphics_expose, 1);
+    G_DEF_SETTERS(ev);
 
     /* GdkEventVisibility */
     ev = gdkevents[GDK_VISIBILITY_NOTIFY];
     DEFINE_ACCESSOR(ev, visibility, state);
+    DEFINE_INIT(ev, visibility);
+    G_DEF_SETTERS(ev);
 
     /* GdkVisibilityState */
     G_DEF_CLASS(GDK_TYPE_VISIBILITY_STATE, "State", ev);
@@ -743,9 +857,10 @@ Init_gtk_gdk_event()
     rb_define_method(ev, "focus?", gdkeventcrossing_focus, 0);
     rb_define_method(ev, "set_focus", gdkeventcrossing_set_focus, 1);
     DEFINE_ACCESSOR(ev, crossing, state);
+    G_DEF_SETTERS(ev);
 
     /* GdkCrossingMode */
-    G_DEF_CLASS(GDK_TYPE_CROSSING_MODE, "CrossingMode", ev);
+    G_DEF_CLASS(GDK_TYPE_CROSSING_MODE, "Mode", ev);
     G_DEF_CONSTANTS(ev, GDK_TYPE_CROSSING_MODE, "GDK_CROSSING_");
     /* GdkNotifyType */
     G_DEF_CLASS(GDK_TYPE_NOTIFY_TYPE, "NotifyType", ev);
@@ -755,6 +870,8 @@ Init_gtk_gdk_event()
     ev = gdkevents[GDK_FOCUS_CHANGE];
     rb_define_method(ev, "in?", gdkeventfocus_change_in, 0);
     rb_define_method(ev, "set_in", gdkeventfocus_change_set_in, 1);
+    DEFINE_INIT(ev, focus_change);
+    G_DEF_SETTERS(ev);
 
     /* GdkEventConfigure */
     ev = gdkevents[GDK_CONFIGURE];
@@ -762,12 +879,16 @@ Init_gtk_gdk_event()
     DEFINE_ACCESSOR(ev, configure, y);
     DEFINE_ACCESSOR(ev, configure, width);
     DEFINE_ACCESSOR(ev, configure, height);
+    DEFINE_INIT(ev, configure);
+    G_DEF_SETTERS(ev);
 
     /* GdkEventProperty */
     ev = gdkevents[GDK_PROPERTY_NOTIFY];
     DEFINE_ACCESSOR(ev, property, atom);
     DEFINE_ACCESSOR(ev, property, time);
     DEFINE_ACCESSOR(ev, property, state);
+    DEFINE_INIT(ev, property);
+    G_DEF_SETTERS(ev);
 
     /* GdkPropertyState */ 
     G_DEF_CLASS(GDK_TYPE_PROPERTY_STATE, "State", ev);
@@ -780,6 +901,7 @@ Init_gtk_gdk_event()
     DEFINE_ACCESSOR(ev, selection, property);
     DEFINE_ACCESSOR(ev, selection, requestor);
     DEFINE_ACCESSOR(ev, selection, time);
+    G_DEF_SETTERS(ev);
 
     /* GdkEventDND */
     ev = gdkevents[GDK_DRAG_ENTER];
@@ -787,11 +909,13 @@ Init_gtk_gdk_event()
     DEFINE_ACCESSOR(ev, dnd, time);
     DEFINE_ACCESSOR(ev, dnd, x_root);
     DEFINE_ACCESSOR(ev, dnd, y_root);
+    G_DEF_SETTERS(ev);
 
     /* GdkEventProximity */
     ev = gdkevents[GDK_PROXIMITY_IN];
     DEFINE_ACCESSOR(ev, proximity, time);
     DEFINE_ACCESSOR(ev, proximity, device);
+    G_DEF_SETTERS(ev);
 
     /* GdkEventClient */
     ev = gdkevents[GDK_CLIENT_EVENT];
@@ -799,17 +923,22 @@ Init_gtk_gdk_event()
     rb_define_method(ev, "data_format", gdkeventclient_data_format, 0);
     rb_define_method(ev, "data", gdkeventclient_data, 0);
     rb_define_method(ev, "send_client_message", 
-					 gdkeventclient_send_client_message, -1);
+                     gdkeventclient_send_client_message, -1);
     rb_define_method(ev, "send_clientmessage_toall", 
-					 gdkeventclient_send_clientmessage_toall, 0);
+                     gdkeventclient_send_clientmessage_toall, 0);
+    DEFINE_INIT(ev, client);
+    G_DEF_SETTERS(ev);
 
     /* GdkEventNoExpose */
-    /* No methods */
+    ev = gdkevents[GDK_NO_EXPOSE];
+    DEFINE_INIT(ev, noexpose);
 
     /* GdkEventWindowState */
     ev = gdkevents[GDK_WINDOW_STATE];
     DEFINE_ACCESSOR(ev, window_state, changed_mask);
     DEFINE_ACCESSOR(ev, window_state, new_window_state);
+    DEFINE_INIT(ev, window_state);
+    G_DEF_SETTERS(ev);
 
     /* GdkWindowState */
     G_DEF_CLASS(GDK_TYPE_WINDOW_STATE, "WindowState", ev);
@@ -819,10 +948,40 @@ Init_gtk_gdk_event()
     ev = gdkevents[GDK_SETTING];
     DEFINE_ACCESSOR(ev, setting, action);
     DEFINE_ACCESSOR(ev, setting, name);
+    DEFINE_INIT(ev, setting);
+    G_DEF_SETTERS(ev);
 
     /* GdkSettingAction */
-    G_DEF_CLASS(GDK_TYPE_SETTING_ACTION, "SettingAction", mGdk);
-    G_DEF_CONSTANTS(mGdk, GDK_TYPE_SETTING_ACTION, "GDK_SETTING_");
+    G_DEF_CLASS(GDK_TYPE_SETTING_ACTION, "Action", ev);
+    G_DEF_CONSTANTS(ev, GDK_TYPE_SETTING_ACTION, "GDK_SETTING_");
+
+#if GTK_CHECK_VERSION(2,6,0)
+    /* GdkEventOwnerChange */
+    ev = gdkevents[GDK_OWNER_CHANGE];
+    DEFINE_ACCESSOR(ev, owner_change, owner);
+    DEFINE_ACCESSOR(ev, owner_change, reason);
+    DEFINE_ACCESSOR(ev, owner_change, selection);
+    DEFINE_ACCESSOR(ev, owner_change, time);
+    DEFINE_ACCESSOR(ev, owner_change, selection_time);
+    G_DEF_SETTERS(ev);
+    DEFINE_INIT(ev, owner_change);
+
+    /* GdkOwnerChange */
+    G_DEF_CLASS(GDK_TYPE_OWNER_CHANGE, "OwnerChange", ev);
+    G_DEF_CONSTANTS(ev, GDK_TYPE_OWNER_CHANGE, "GDK_OWNER_CHANGE_");
+#endif
+
+#if GTK_CHECK_VERSION(2,8,0)
+    /* GdkEventGrabBroken */
+    ev = gdkevents[GDK_GRAB_BROKEN];
+    rb_define_method(ev, "keyboard?", gdkeventgrab_broken_keyboard, 0);
+    rb_define_method(ev, "set_keyboard", gdkeventgrab_broken_set_keyboard, 1);
+    rb_define_method(ev, "implicit?", gdkeventgrab_broken_implicit, 0);
+    rb_define_method(ev, "set_implicit", gdkeventgrab_broken_set_implicit, 1);
+    DEFINE_ACCESSOR(ev, grab_broken, grab_window);
+    DEFINE_INIT(ev, grab_broken);
+    G_DEF_SETTERS(ev);
+#endif
 
     rbgobj_register_g2r_func(GDK_TYPE_EVENT, &gdkevent_g2r);
 
@@ -831,6 +990,7 @@ Init_gtk_gdk_event()
      */
     rb_define_const(gdkEvent, "CURRENT_TIME", INT2FIX(GDK_CURRENT_TIME));
     rb_define_const(gdkEvent, "PRIORITY_EVENTS", INT2FIX(GDK_PRIORITY_EVENTS));
+    rb_define_const(gdkEvent, "PRIORITY_REDRAW", INT2FIX(GDK_PRIORITY_REDRAW));
 
     /* GdkEventType */
     /* XXX */
@@ -844,4 +1004,5 @@ Init_gtk_gdk_event()
     /* GdkEventMask */
     G_DEF_CLASS(GDK_TYPE_EVENT_MASK, "Mask", gdkEvent);
     G_DEF_CONSTANTS(gdkEvent, GDK_TYPE_EVENT_MASK, "GDK_");
+
 }
