@@ -4,9 +4,9 @@
   rbgtkcontainer.c -
 
   $Author: mutoh $
-  $Date: 2005/11/06 04:44:24 $
+  $Date: 2006/06/17 06:59:32 $
 
-  Copyright (C) 2002-2005 Ruby-GNOME2 Project Team
+  Copyright (C) 2002-2006 Ruby-GNOME2 Project Team
   Copyright (C) 1998-2000 Yukihiro Matsumoto,
                           Daisuke Kanda,
                           Hiroshi Igarashi
@@ -84,7 +84,7 @@ static VALUE
 cont_get_children(self)
     VALUE self;
 {
-   return GLIST2ARY(gtk_container_get_children(_SELF(self)));
+   return GLIST2ARYF(gtk_container_get_children(_SELF(self)));
 }
 
 static VALUE
@@ -369,16 +369,8 @@ cont_get_focus_chain(self)
 {
     gboolean ret;
     GList *glist = NULL;
-    VALUE result;
     ret = gtk_container_get_focus_chain(_SELF(self), &glist);
-
-    if (ret){
-        result = GLIST2ARY(glist);
-        g_list_free(glist);
-    } else {
-        result = Qnil;
-    }
-    return result;
+    return GLIST2ARYF(glist);
 }
 
 static VALUE
@@ -720,11 +712,24 @@ type_register(int argc, VALUE* argv, VALUE self)
 }
 /**********************************************************************/
 
+static void
+cont_mark_callback(GtkWidget* w, gpointer data)
+{
+    rbgobj_gc_mark_instance(w);
+}
+
+static void
+cont_mark(void* p)
+{
+    gtk_container_forall(GTK_CONTAINER(p), cont_mark_callback, NULL);
+}
+
+/**********************************************************************/
 
 void 
 Init_gtk_container()
 {
-    VALUE gContainer = G_DEF_CLASS(GTK_TYPE_CONTAINER, "Container", mGtk);
+    VALUE gContainer = G_DEF_CLASS2(GTK_TYPE_CONTAINER, "Container", mGtk, cont_mark, NULL);
 
     rb_define_method(gContainer, "resize_container?", cont_is_resize_container, 0);
     rb_define_method(gContainer, "add", cont_add, -1);
