@@ -3,10 +3,10 @@
 
   rbgtktreeview.c -
 
-  $Author: sakai $
-  $Date: 2003/11/20 18:27:54 $
+  $Author: ktou $
+  $Date: 2006/12/05 14:54:28 $
 
-  Copyright (C) 2002,2003 Masao Mutoh
+  Copyright (C) 2002-2005 Masao Mutoh
 ************************************************/
 
 #include "global.h"
@@ -19,6 +19,9 @@
 #define RVAL2ITR(i) ((GtkTreeIter*)RVAL2BOXED(i, GTK_TYPE_TREE_ITER))
 #define RVAL2CELLRENDERER(c) (GTK_CELL_RENDERER(RVAL2GOBJ(c)))
 
+static ID id_model;
+static ID id_selection;
+
 static VALUE
 treeview_initialize(argc, argv, self)
     int argc;
@@ -29,6 +32,7 @@ treeview_initialize(argc, argv, self)
     GtkWidget *widget;
 
     if (rb_scan_args(argc, argv, "01", &model) == 1) {
+        G_CHILD_SET(self, id_model, model);
         widget = gtk_tree_view_new_with_model(GTK_TREE_MODEL(RVAL2GOBJ(model)));
     }
     else {
@@ -43,7 +47,9 @@ static VALUE
 treeview_get_selection(self)
     VALUE self;
 {
-    return GOBJ2RVAL(gtk_tree_view_get_selection(_SELF(self)));
+    VALUE ret = GOBJ2RVAL(gtk_tree_view_get_selection(_SELF(self)));
+    G_CHILD_SET(self, id_selection, ret);
+    return ret;
 }
 
 static VALUE
@@ -58,6 +64,7 @@ static VALUE
 treeview_append_column(self, column)
     VALUE self, column;
 {
+    G_CHILD_ADD(self, column);
     return INT2NUM(gtk_tree_view_append_column(_SELF(self), 
                                                TREEVIEW_COL(column)));
 }
@@ -66,6 +73,7 @@ static VALUE
 treeview_remove_column(self, column)
     VALUE self, column;
 {
+    G_CHILD_REMOVE(self, column);
     return INT2NUM(gtk_tree_view_remove_column(_SELF(self), 
                                                TREEVIEW_COL(column)));
 }  
@@ -99,6 +107,7 @@ treeview_insert_column(argc, argv, self)
                  &args[3]);        /*           attributes            */
 
     if (argc == 2) {
+        G_CHILD_ADD(self, args[0]);
         return INT2NUM(gtk_tree_view_insert_column(_SELF(self),
                                                    TREEVIEW_COL(args[0]),
                                                    NUM2INT(args[1])));
@@ -111,7 +120,7 @@ treeview_insert_column(argc, argv, self)
                                                          NUM2INT(args[0]),
                                                          RVAL2CSTR(args[1]),
                                                          RVAL2CELLRENDERER(args[2]),
-                                                         cell_data_func,
+                                                         (GtkTreeCellDataFunc)cell_data_func,
                                                          (gpointer)func,
                                                          NULL);
         return INT2NUM(ret);
@@ -126,6 +135,10 @@ treeview_insert_column(argc, argv, self)
         GtkTreeViewColumn* column = gtk_tree_view_column_new();
 
         Check_Type(args[3], T_HASH);
+
+        G_CHILD_ADD(self, args[2]);
+        G_CHILD_ADD(self, args[3]);
+
         gtk_tree_view_column_set_title(column, RVAL2CSTR(args[1]));
         gtk_tree_view_column_pack_start(column, renderer, TRUE);
         
@@ -162,7 +175,7 @@ static VALUE
 treeview_get_columns(self)
     VALUE self;
 {
-    return GLIST2ARY(gtk_tree_view_get_columns(_SELF(self)));
+    return GLIST2ARYF(gtk_tree_view_get_columns(_SELF(self)));
 }
 
 static VALUE
@@ -170,7 +183,7 @@ treeview_move_column_after(self, column, base_column)
     VALUE self, column, base_column;
 {
     gtk_tree_view_move_column_after(_SELF(self), TREEVIEW_COL(column),
-                                    TREEVIEW_COL(base_column));
+                                    NIL_P(base_column) ? NULL : TREEVIEW_COL(base_column));
     return self;
 }
 
@@ -211,8 +224,10 @@ static VALUE
 treeview_scroll_to_cell(self, path, column, use_align, row_align, col_align)
     VALUE self, path, column, use_align, row_align, col_align;
 {
-    gtk_tree_view_scroll_to_cell(_SELF(self), RVAL2TREEPATH(path),
-                                 TREEVIEW_COL(column), RTEST(use_align),
+    gtk_tree_view_scroll_to_cell(_SELF(self),
+                                 NIL_P(path) ? NULL : RVAL2TREEPATH(path),
+                                 NIL_P(column) ? NULL : TREEVIEW_COL(column), 
+                                 RTEST(use_align),
                                  NUM2DBL(row_align), NUM2DBL(col_align));
     return self;
 }
@@ -222,7 +237,8 @@ treeview_set_cursor(self, path, focus_column, start_editing)
     VALUE self, path, focus_column, start_editing;
 {
     gtk_tree_view_set_cursor(_SELF(self), RVAL2TREEPATH(path),
-                             TREEVIEW_COL(focus_column), RTEST(start_editing));
+                             NIL_P(focus_column) ? NULL : TREEVIEW_COL(focus_column), 
+                             RTEST(start_editing));
     return self;
 }
 
@@ -234,8 +250,8 @@ treeview_get_cursor(self)
     GtkTreeViewColumn* focus_column;
 
     gtk_tree_view_get_cursor(_SELF(self), &path, &focus_column);
-    return rb_ary_new3(2, (path == NULL) ? Qnil : TREEPATH2RVAL(path),
-                       (focus_column == NULL) ? Qnil : GOBJ2RVAL(focus_column));
+    return rb_ary_new3(2, path ? TREEPATH2RVAL(path) : Qnil,
+                       GOBJ2RVAL(focus_column));
 }
 
 static VALUE
@@ -333,8 +349,8 @@ treeview_get_path_at_pos(self, x, y)
                                         NUM2INT(x), NUM2INT(y),
                                         &path, &column, &cell_x, &cell_y);
     return ret ? rb_ary_new3(4, 
-                             (path == NULL) ? Qnil : TREEPATH2RVAL(path),
-                             (column == NULL) ? Qnil : GOBJ2RVAL(column),
+                             path ? TREEPATH2RVAL(path) : Qnil,
+                             column ? GOBJ2RVAL(column) : Qnil,
                              INT2NUM(cell_x), INT2NUM(cell_y)) : Qnil;
 }
 
@@ -343,8 +359,10 @@ treeview_get_cell_area(self, path, column)
     VALUE self, path, column;
 {
     GdkRectangle rect;
-    gtk_tree_view_get_cell_area(_SELF(self), RVAL2TREEPATH(path),
-                                TREEVIEW_COL(column), &rect);
+    gtk_tree_view_get_cell_area(_SELF(self), 
+                                NIL_P(path) ? NULL : RVAL2TREEPATH(path),
+                                NIL_P(column) ? NULL : TREEVIEW_COL(column), 
+                                &rect);
     return BOXED2RVAL(&rect, GDK_TYPE_RECTANGLE);
 }
 
@@ -354,8 +372,9 @@ treeview_get_background_area(self, path, column)
 {
     GdkRectangle rect;
     gtk_tree_view_get_background_area(_SELF(self), 
-                                      RVAL2TREEPATH(path),
-                                      TREEVIEW_COL(column), &rect);
+                                      NIL_P(path) ? NULL : RVAL2TREEPATH(path),
+                                      NIL_P(column) ? NULL : TREEVIEW_COL(column), 
+                                      &rect);
     return BOXED2RVAL(&rect, GDK_TYPE_RECTANGLE);
 }
 
@@ -368,6 +387,21 @@ treeview_get_visible_rect(self)
     return BOXED2RVAL(&rect, GDK_TYPE_RECTANGLE);
 }
 
+#if GTK_CHECK_VERSION(2,8,0)
+static VALUE
+treeview_get_visible_range(self)
+    VALUE self;
+{
+    GtkTreePath* start_path;
+    GtkTreePath* end_path;
+
+    gboolean valid_paths = gtk_tree_view_get_visible_range(_SELF(self), &start_path, &end_path);
+
+    return valid_paths ? rb_assoc_new(TREEPATH2RVAL(start_path),
+                                      TREEPATH2RVAL(end_path)) : Qnil;
+}
+#endif
+
 static VALUE
 treeview_get_bin_window(self)
     VALUE self;
@@ -445,7 +479,8 @@ static VALUE
 treeview_set_drag_dest_row(self, path, pos)
     VALUE self, path, pos;
 {
-    gtk_tree_view_set_drag_dest_row(_SELF(self), RVAL2TREEPATH(path),
+    gtk_tree_view_set_drag_dest_row(_SELF(self), 
+                                    NIL_P(path)?NULL:RVAL2TREEPATH(path),
                                     RVAL2GENUM(pos, GTK_TYPE_TREE_VIEW_DROP_POSITION));
     return self;
 }
@@ -454,10 +489,10 @@ static VALUE
 treeview_get_drag_dest_row(self)
     VALUE self;
 {
-    GtkTreePath* path;
+    GtkTreePath* path = NULL;
     GtkTreeViewDropPosition pos;
     gtk_tree_view_get_drag_dest_row(_SELF(self), &path, &pos);
-    return rb_ary_new3(2, TREEPATH2RVAL(path), 
+    return rb_ary_new3(2, path ? TREEPATH2RVAL(path) : Qnil, 
                        GENUM2RVAL(pos, GTK_TYPE_TREE_VIEW_DROP_POSITION));
 }
 
@@ -472,7 +507,7 @@ treeview_get_dest_row_at_pos(self, drag_x, drag_y)
     ret = gtk_tree_view_get_dest_row_at_pos(_SELF(self), 
                                             NUM2INT(drag_x), NUM2INT(drag_y),
                                             &path, &pos);
-    return ret ? rb_ary_new3(2, TREEPATH2RVAL(path), 
+    return ret ? rb_ary_new3(2, path ? TREEPATH2RVAL(path) : Qnil, 
                              GENUM2RVAL(pos, GTK_TYPE_TREE_VIEW_DROP_POSITION)) : Qnil;
 }
 
@@ -539,17 +574,167 @@ treeview_set_cursor_on_cell(self, path, focus_column, focus_cell, start_editing)
     VALUE self, path, focus_column, focus_cell, start_editing;
 {
     gtk_tree_view_set_cursor_on_cell(_SELF(self), RVAL2TREEPATH(path),
-                             TREEVIEW_COL(focus_column), GTK_CELL_RENDERER(RVAL2GOBJ(focus_cell)), 
-                             RTEST(start_editing));
+                                     NIL_P(focus_column) ? NULL : TREEVIEW_COL(focus_column), 
+                                     NIL_P(focus_cell) ? NULL : GTK_CELL_RENDERER(RVAL2GOBJ(focus_cell)), 
+                                     RTEST(start_editing));
     return self;
 }
 #endif
 
+/* Defined as Property.
+gboolean    gtk_tree_view_get_fixed_height_mode
+                                            (GtkTreeView *tree_view);
+void        gtk_tree_view_set_fixed_height_mode
+                                            (GtkTreeView *tree_view,
+                                             gboolean enable);
+gboolean    gtk_tree_view_get_hover_selection
+                                            (GtkTreeView *tree_view);
+void        gtk_tree_view_set_hover_selection
+                                            (GtkTreeView *tree_view,
+                                             gboolean hover);
+gboolean    gtk_tree_view_get_hover_expand  (GtkTreeView *tree_view);
+void        gtk_tree_view_set_hover_expand  (GtkTreeView *tree_view,
+                                             gboolean expand);
+*/
+
+/* How can I implement this?
+GtkTreeViewRowSeparatorFunc gtk_tree_view_get_row_separator_func
+                                            (GtkTreeView *tree_view);
+*/
+
+#if GTK_CHECK_VERSION(2,6,0)
+static gboolean
+row_separator_func(model, iter, func)
+    GtkTreeModel* model;
+    GtkTreeIter* iter;
+    gpointer* func;
+{
+    VALUE ret;
+    iter->user_data3 = model;
+    ret = rb_funcall((VALUE)func, id_call, 2, GOBJ2RVAL(model), 
+                     BOXED2RVAL(iter, GTK_TYPE_TREE_ITER));
+    return CBOOL2RVAL(ret);
+}
+
+static VALUE
+treeview_set_row_separator_func(self)
+    VALUE self;
+{
+    VALUE func = G_BLOCK_PROC();
+
+    G_RELATIVE(self, func);
+    gtk_tree_view_set_row_separator_func(_SELF(self), 
+                                         (GtkTreeViewRowSeparatorFunc)row_separator_func, 
+                                         (gpointer)func, 
+                                         (GtkDestroyNotify)NULL);
+    return self;
+}
+#endif
+
+#if GTK_CHECK_VERSION(2,10,0)
+static VALUE
+treeview_get_search_entry(self)
+    VALUE self;
+{
+    return GOBJ2RVAL(gtk_tree_view_get_search_entry(_SELF(self)));
+}
+
+static VALUE
+treeview_set_search_entry(self, entry)
+    VALUE self, entry;
+{
+    gtk_tree_view_set_search_entry(_SELF(self), RVAL2GOBJ(entry));
+    return self;
+}
+
+/* Can't define this.
+GtkTreeViewSearchPositionFunc gtk_tree_view_get_search_position_func
+                                            (GtkTreeView *tree_view);
+*/
+
+struct callback_arg
+{
+    VALUE callback;
+    VALUE tree_view;
+    VALUE search_dialog;
+};
+
+static VALUE
+invoke_callback(VALUE data)
+{
+    struct callback_arg *arg = (struct callback_arg *)data;
+    rb_funcall(arg->callback, id_call, 2, arg->tree_view, arg->search_dialog);
+    return Qnil;
+}
+
+static void
+search_position_func(tree_view, search_dialog, func)
+    GtkTreeView* tree_view;
+    GtkWidget* search_dialog;
+    gpointer func;
+{
+    struct callback_arg arg;
+
+    arg.callback = (VALUE)func;
+    arg.tree_view = GOBJ2RVAL(tree_view);
+    arg.search_dialog = GOBJ2RVAL(search_dialog);
+    G_PROTECT_CALLBACK(invoke_callback, &arg);
+}
+
+static void
+remove_callback_reference(gpointer data)
+{
+    G_CHILD_REMOVE(mGtk, (VALUE)data);
+}
+
+
+static VALUE
+treeview_set_search_position_func(self)
+    VALUE self;
+{
+    VALUE func = G_BLOCK_PROC();
+    G_CHILD_ADD(mGtk, func);
+    gtk_tree_view_set_search_position_func(_SELF(self),
+                                           (GtkTreeViewSearchPositionFunc)search_position_func,
+                                           (gpointer)func,
+                                           (GDestroyNotify)remove_callback_reference);
+    return self;
+}
+
+#endif
+/* Defined as properties
+gboolean    gtk_tree_view_get_headers_clickable
+                                            (GtkTreeView *tree_view);
+void        gtk_tree_view_set_headers_clickable
+                                            (GtkTreeView *tree_view,
+                                             gboolean setting);
+gboolean    gtk_tree_view_get_rubber_banding
+                                            (GtkTreeView *tree_view);
+void        gtk_tree_view_set_rubber_banding
+                                            (GtkTreeView *tree_view,
+                                             gboolean enable);
+gboolean    gtk_tree_view_get_enable_tree_lines
+                                            (GtkTreeView *tree_view);
+void        gtk_tree_view_set_enable_tree_lines
+                                            (GtkTreeView *tree_view,
+                                             gboolean enabled);
+
+// Use Gtk::TreeView#enable_grid_lines, #set_enable_grid_lines instead.
+GtkTreeViewGridLines gtk_tree_view_get_grid_lines
+                                            (GtkTreeView *tree_view);
+
+void        gtk_tree_view_set_grid_lines    (GtkTreeView *tree_view,
+                                             GtkTreeViewGridLines grid_lines);
+*/
+
 void 
 Init_gtk_treeview()
 {
     VALUE gTv = G_DEF_CLASS(GTK_TYPE_TREE_VIEW, "TreeView", mGtk);
-    
+
+    id_selection = rb_intern("selection");
+    id_model = rb_intern("model");
+
     rb_define_method(gTv, "initialize", treeview_initialize, -1);
     rb_define_method(gTv, "selection", treeview_get_selection, 0);
     rb_define_method(gTv, "columns_autosize", treeview_columns_autosize, 0);
@@ -569,15 +754,20 @@ Init_gtk_treeview()
     rb_define_method(gTv, "collapse_all", treeview_collapse_all, 0);
     rb_define_method(gTv, "expand_row", treeview_expand_row, 2);
     rb_define_method(gTv, "collapse_row", treeview_collapse_row, 1);
-   #if GTK_CHECK_VERSION(2,2,0)
+#if GTK_CHECK_VERSION(2,2,0)
     rb_define_method(gTv, "expand_to_path", treeview_expand_to_path, 1);
-   #endif
+#endif
     rb_define_method(gTv, "map_expanded_rows", treeview_map_expanded_rows, 0);
     rb_define_method(gTv, "row_expanded?", treeview_row_expanded, 1);
     rb_define_method(gTv, "get_path_at_pos", treeview_get_path_at_pos, 2);
+    rb_define_alias(gTv, "get_path", "get_path_at_pos");
     rb_define_method(gTv, "get_cell_area", treeview_get_cell_area, 2);
     rb_define_method(gTv, "get_background_area", treeview_get_background_area, 2);
     rb_define_method(gTv, "visible_rect", treeview_get_visible_rect, 0);
+#if GTK_CHECK_VERSION(2,8,0)
+    rb_define_method(gTv, "visible_range", treeview_get_visible_range, 0);
+#endif
+
     rb_define_method(gTv, "bin_window", treeview_get_bin_window, 0);
     rb_define_method(gTv, "widget_to_tree_coords", treeview_widget_to_tree_coords, 2);
     rb_define_method(gTv, "tree_to_widget_coords", treeview_tree_to_widget_coords, 2);
@@ -588,19 +778,35 @@ Init_gtk_treeview()
     rb_define_method(gTv, "set_drag_dest_row", treeview_set_drag_dest_row, 2);
     rb_define_method(gTv, "drag_dest_row", treeview_get_drag_dest_row, 0);
     rb_define_method(gTv, "get_dest_row_at_pos", treeview_get_dest_row_at_pos, 2);
+    rb_define_alias(gTv, "get_dest_row", "get_dest_row_at_pos");
     rb_define_method(gTv, "create_row_drag_icon", treeview_create_row_drag_icon, 1);
     rb_define_method(gTv, "set_search_equal_func", treeview_set_search_equal_func, 0);
 #if GTK_CHECK_VERSION(2,2,0)
     rb_define_method(gTv, "set_cursor_on_cell", treeview_set_cursor_on_cell, 4);
 #endif
-    
+
+#if GTK_CHECK_VERSION(2,6,0)
+    rb_define_method(gTv, "set_row_separator_func", treeview_set_row_separator_func, 0);
+#endif
+#if GTK_CHECK_VERSION(2,10,0)
+    rb_define_method(gTv, "search_entry", treeview_get_search_entry, 0);
+    rb_define_method(gTv, "set_search_entry", treeview_set_search_entry, 1);
+    rb_define_method(gTv, "set_search_position_func", treeview_set_search_position_func, 0);
+#endif
+
     /* Constants */
     G_DEF_CLASS(GTK_TYPE_TREE_VIEW_DROP_POSITION, "DropPosition", gTv);
     G_DEF_CONSTANTS(gTv, GTK_TYPE_TREE_VIEW_DROP_POSITION, "GTK_TREE_VIEW_");
 
+#if GTK_CHECK_VERSION(2,10,0)
+    /* GtkTreeViewGridLines */
+    G_DEF_CLASS(GTK_TYPE_TREE_VIEW_GRID_LINES, "GridLines", gTv);
+    G_DEF_CONSTANTS(gTv, GTK_TYPE_TREE_VIEW_GRID_LINES, "GTK_TREE_VIEW_");
+#endif
+
     /* Option Signals */
-    G_DEF_SIGNAL_FUNC(gTv, "row-collapsed", treeview_signal_func);
-    G_DEF_SIGNAL_FUNC(gTv, "row-expanded", treeview_signal_func);
-    G_DEF_SIGNAL_FUNC(gTv, "test-collapse-row", treeview_signal_func);
-    G_DEF_SIGNAL_FUNC(gTv, "test-expand-row", treeview_signal_func);
+    G_DEF_SIGNAL_FUNC(gTv, "row-collapsed", (GValToRValSignalFunc)treeview_signal_func);
+    G_DEF_SIGNAL_FUNC(gTv, "row-expanded", (GValToRValSignalFunc)treeview_signal_func);
+    G_DEF_SIGNAL_FUNC(gTv, "test-collapse-row", (GValToRValSignalFunc)treeview_signal_func);
+    G_DEF_SIGNAL_FUNC(gTv, "test-expand-row", (GValToRValSignalFunc)treeview_signal_func);
 }
