@@ -7,7 +7,6 @@
 #
 
 require 'gtkglext'
-require 'pango'
 
 FONT_STRING = "courier 12"
 
@@ -57,11 +56,7 @@ def realize(w)
             exit 1
         end
 
-        begin
-            font_metrics = font.metrics
-        rescue TypeError, ArgumentError # the error depend on if I give no or a nil arg to font.metrics
-            raise "Sorry, but this example does not work with Ruby-GNOME2 0.6.0 because of a bug in Ruby/Pango."
-        end
+        font_metrics = font.metrics
         $font_height = font_metrics.ascent + font_metrics.descent
         $font_height = Pango.pixels($font_height)
 
@@ -81,8 +76,6 @@ def realize(w)
     end
 end
 
-# Init GTK
-Gtk.init
 # Init GtkGlExt
 Gtk::GL.init
 # Query OpenGL extension version
@@ -108,13 +101,15 @@ end
 examine_gl_config_attrib(glconfig)
 
 # Top-level window
-window = Gtk::Window.new
-window.title = "font"
+window = Gtk::Window.new("font")
 # Perform the resize immediately
 window.resize_mode = Gtk::RESIZE_IMMEDIATE
 # Get automatically redrawn if any of their children changed allocation.
 window.reallocate_redraws = true
-window.signal_connect("delete_event") { Gtk.main_quit }
+window.signal_connect("delete_event") do
+    Gtk.main_quit
+    true
+end
 
 # VBox
 vbox = Gtk::VBox.new
@@ -180,7 +175,6 @@ drawing_area.show
 # Simple quit button
 button = Gtk::Button.new("Quit")
 button.signal_connect("clicked") do
-    window.destroy
     Gtk.main_quit
 end
 vbox.pack_start(button, false, false)
